# Beelzebub ![CI](https://github.com/mariocandela/beelzebub/actions/workflows/ci.yml/badge.svg) ![Docker](https://github.com/mariocandela/beelzebub/actions/workflows/docker-image.yml/badge.svg)

[![logo-1.png](https://i.postimg.cc/KvbsJFp3/logo-1.png)](https://postimg.cc/yWfPNqH7)

A secure honeypot framework, extremely easy to configure by yaml 🚀

## Examples: 

[`mariocandela/beelzebub-example`](https://github.com/mariocandela/beelzebub-example)

## Quick Start

Using [`docker-compose`](https://docs.docker.com/compose/)

```bash
$ docker-compose build
$ docker-compose up -d
 ```

Using [`go compiler`](https://go.dev/doc/install)

```bash
$ go mod download
$ go build 
$ ./beelzebub
 ```

Unit Test:

```bash
$ go test ./...
 ```

## Example configuration service 

The configurations are inside the /configurations/services directory, just add a new file for each service/port.

### Example HTTP Honeypot on 80 port

###### http-80.yaml

```yaml
apiVersion: "v1"
protocol: "http"
address: ":80"
description: "Wordpress 6.0"
commands:
  - regex: "index.php"
    handler: "<!DOCTYPE html><html lang=\"en-US\"><head><meta charset=\"UTF-8\" /><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\" /><meta name='robots' content='max-image-preview:large' /><title>test – Just another WordPress site</title><link rel='dns-prefetch' href='//s.w.org' /><link rel=\"alternate\" type=\"application/rss+xml\" title=\"test » Feed\" href=\"https://wordpress.com/?feed=rss2\" /><link rel=\"alternate\" type=\"application/rss+xml\" title=\"test » Comments Feed\" href=\"https://wordpress.com/?feed=comments-rss2\" /><script>;window._wpemojiSettings={'baseUrl':'https:\\/\\/s.w.org\\/images\\/core\\/emoji\\/14.0.0\\/72x72\\/','ext':'.png','svgUrl':'https:\\/\\/s.w.org\\/images\\/core\\/emoji\\/14.0.0\\/svg\\/','svgExt':'.svg','source':{'concatemoji':'http:\\/\\/X\\/wp-includes\\/js\\/wp-emoji-release.min.js?ver=6.0'}};/*! This file is auto-generated */!function(t,a,e){var s,r,i,n=a.createElement('canvas'),o=n.getContext&&n.getContext('2d');function p(e,t){var a=String.fromCharCode,e=(o.clearRect(0,0,n.width,n.height),o.fillText(a.apply(this,e),0,0),n.toDataURL());return o.clearRect(0,0,n.width,n.height),o.fillText(a.apply(this,t),0,0),e===n.toDataURL()};function c(e){var t=a.createElement('script');t.src=e,t.defer=t.type='text/javascript',a.getElementsByTagName('head')[0].appendChild(t)};for(i=Array('flag','emoji'),e.supports={everything:!0,everythingExceptFlag:!0},r=0;r<i.length;r++)e.supports[i[r]]=function(e){if(!o||!o.fillText)return!1;switch(o.textBaseline='top',o.font='600 32px Arial',e){case'flag':return p([127987,65039,8205,9895,65039],[127987,65039,8203,9895,65039])?!1:!p([55356,56826,55356,56819],[55356,56826,8203,55356,56819])&&!p([55356,57332,56128,56423,56128,56418,56128,56421,56128,56430,56128,56423,56128,56447],[55356,57332,8203,56128,56423,8203,56128,56418,8203,56128,56421,8203,56128,56430,8203,56128,56423,8203,56128,56447]);case'emoji':return!p([129777,127995,8205,129778,127999],[129777,127995,8203,129778,127999])};return!1}(i[r]),e.supports.everything=e.supports.everything&&e.supports[i[r]],'flag'!==i[r]&&(e.supports.everythingExceptFlag=e.supports.everythingExceptFlag&&e.supports[i[r]]);e.supports.everythingExceptFlag=e.supports.everythingExceptFlag&&!e.supports.flag,e.DOMReady=!1,e.readyCallback=function(){e.DOMReady=!0},e.supports.everything||(s=function(){e.readyCallback()},a.addEventListener?(a.addEventListener('DOMContentLoaded',s,!1),t.addEventListener('load',s,!1)):(t.attachEvent('onload',s),a.attachEvent('onreadystatechange',function(){'complete'===a.readyState&&e.readyCallback()})),(t=e.source||{}).concatemoji?c(t.concatemoji):t.wpemoji&&t.twemoji&&(c(t.twemoji),c(t.wpemoji)))}(window,document,window._wpemojiSettings);</script><style>img.wp-smiley,img.emoji{display:inline !important;border:none !important;box-shadow:none !important;height:1em !important;width:1em !important;margin:0 0.07em !important;vertical-align:-0.1em !important;background:none !important;padding:0 !important}</style><style id='wp-block-site-logo-inline-css'>.wp-block-site-logo{line-height:0}.wp-block-site-logo a{display:inline-block}.wp-block-site-logo.is-default-size img{width:120px;height:auto}.wp-block-site-logo a,.wp-block-site-logo img{border-radius:inherit}.wp-block-site-logo.aligncenter{margin-left:auto;margin-right:auto;text-align:center}.wp-block-site-logo.is-style-rounded{border-radius:9999px}</style><style id='wp-block-group-inline-css'>.wp-block-group{box-sizing:border-box}:where(.wp-block-group.has-background){padding:1.25em 2.375em}</style><style id='wp-block-page-list-inline-css'>.wp-block-navigation .wp-block-page-list{display:flex;background-color:inherit}.wp-block-navigation .wp-block-navigation-item{background-color:inherit}</style><link rel='stylesheet' id='wp-block-navigation-css' href='https://wordpress.com/wp-includes/blocks/navigation/style.min.css?ver=6.0' media='all' /><style id='wp-block-template-part-inline-css'>.wp-block-template-part.has-background{padding:1.25em 2.375em;margin-top:0;margin-bottom:0}</style><style id='wp-block-image-inline-css'>.wp-block-image{margin:0 0 1em}.wp-block-image img{height:auto;max-width:100%;vertical-align:bottom}.wp-block-image:not(.is-style-rounded)>a,.wp-block-image:not(.is-style-rounded) img{border-radius:inherit}.wp-block-image.aligncenter{text-align:center}.wp-block-image.alignfull img,.wp-block-image.alignwide img{height:auto;width:100%}.wp-block-image.aligncenter,.wp-block-image .aligncenter,.wp-block-image.alignleft,.wp-block-image .alignleft,.wp-block-image.alignright,.wp-block-image .alignright{display:table}.wp-block-image.aligncenter>figcaption,.wp-block-image .aligncenter>figcaption,.wp-block-image.alignleft>figcaption,.wp-block-image .alignleft>figcaption,.wp-block-image.alignright>figcaption,.wp-block-image .alignright>figcaption{display:table-caption;caption-side:bottom}.wp-block-image .alignleft{float:left;margin:.5em 1em .5em 0}.wp-block-image .alignright{float:right;margin:.5em 0 .5em 1em}.wp-block-image .aligncenter{margin-left:auto;margin-right:auto}.wp-block-image figcaption{margin-top:.5em;margin-bottom:1em}.wp-block-image.is-style-circle-mask img,.wp-block-image.is-style-rounded img,.wp-block-image .is-style-rounded img{border-radius:9999px}@supports ((-webkit-mask-image:none) or (mask-image:none)) or (-webkit-mask-image:none){.wp-block-image.is-style-circle-mask img{-webkit-mask-image:url('data:image/svg+xml;utf8,<svg viewBox=\"0 0 100 100\" xmlns=\"http://www.w3.org/2000/svg\"><circle cx=\"50\" cy=\"50\" r=\"50\"/></svg>');mask-image:url('data:image/svg+xml;utf8,<svg viewBox=\"0 0 100 100\" xmlns=\"http://www.w3.org/2000/svg\"><circle cx=\"50\" cy=\"50\" r=\"50\"/></svg>');mask-mode:alpha;-webkit-mask-repeat:no-repeat;mask-repeat:no-repeat;-webkit-mask-size:contain;mask-size:contain;-webkit-mask-position:center;mask-position:center;border-radius:0}}.wp-block-image figure{margin:0}.wp-block-image figcaption{color:#555;font-size:13px;text-align:center}.is-dark-theme .wp-block-image figcaption{color:hsla(0,0%,100%,.65)}</style><style id='wp-block-spacer-inline-css'>.wp-block-spacer{clear:both}</style><style id='wp-block-post-title-inline-css'>.wp-block-post-title{word-break:break-word}.wp-block-post-title a{display:inline-block}</style><style id='wp-block-post-featured-image-inline-css'>.wp-block-post-featured-image{margin-left:0;margin-right:0}.wp-block-post-featured-image a{display:block}.wp-block-post-featured-image img{max-width:100%;width:100%;height:auto;vertical-align:bottom}.wp-block-post-featured-image.alignfull img,.wp-block-post-featured-image.alignwide img{width:100%}</style><style id='wp-block-paragraph-inline-css'>.is-small-text{font-size:.875em}.is-regular-text{font-size:1em}.is-large-text{font-size:2.25em}.is-larger-text{font-size:3em}.has-drop-cap:not(:focus):first-letter{float:left;font-size:8.4em;line-height:.68;font-weight:100;margin:.05em .1em 0 0;text-transform:uppercase;font-style:normal}p.has-drop-cap.has-background{overflow:hidden}p.has-background{padding:1.25em 2.375em}:where(p.has-text-color:not(.has-link-color)) a{color:inherit}</style><style id='wp-block-post-excerpt-inline-css'>.wp-block-post-excerpt__more-link{display:inline-block}</style><style id='wp-block-columns-inline-css'> .wp-block-columns{display:flex;margin-bottom:1.75em;box-sizing:border-box;flex-wrap:wrap!important;align-items:normal!important}@media (min-width:782px){.wp-block-columns{flex-wrap:nowrap!important}}.wp-block-columns.are-vertically-aligned-top{align-items:flex-start}.wp-block-columns.are-vertically-aligned-center{align-items:center}.wp-block-columns.are-vertically-aligned-bottom{align-items:flex-end}@media (max-width:781px){.wp-block-columns:not(.is-not-stacked-on-mobile)>.wp-block-column{flex-basis:100%!important}}@media (min-width:782px){.wp-block-columns:not(.is-not-stacked-on-mobile)>.wp-block-column{flex-basis:0;flex-grow:1}.wp-block-columns:not(.is-not-stacked-on-mobile)>.wp-block-column[style*=flex-basis]{flex-grow:0}}.wp-block-columns.is-not-stacked-on-mobile{flex-wrap:nowrap!important}.wp-block-columns.is-not-stacked-on-mobile>.wp-block-column{flex-basis:0;flex-grow:1}.wp-block-columns.is-not-stacked-on-mobile>.wp-block-column[style*=flex-basis]{flex-grow:0}:where(.wp-block-columns.has-background){padding:1.25em 2.375em}.wp-block-column{flex-grow:1;min-width:0;word-break:break-word;overflow-wrap:break-word}.wp-block-column.is-vertically-aligned-top{align-self:flex-start}.wp-block-column.is-vertically-aligned-center{align-self:center}.wp-block-column.is-vertically-aligned-bottom{align-self:flex-end}.wp-block-column.is-vertically-aligned-bottom,.wp-block-column.is-vertically-aligned-center,.wp-block-column.is-vertically-aligned-top{width:100%} </style><style id='wp-block-post-template-inline-css'>.wp-block-post-template,.wp-block-query-loop{margin-top:0;margin-bottom:0;max-width:100%;list-style:none;padding:0}.wp-block-post-template.wp-block-post-template,.wp-block-query-loop.wp-block-post-template{background:none}.wp-block-post-template.is-flex-container,.wp-block-query-loop.is-flex-container{flex-direction:row;display:flex;flex-wrap:wrap;gap:1.25em}.wp-block-post-template.is-flex-container li,.wp-block-query-loop.is-flex-container li{margin:0;width:100%}@media (min-width:600px){.wp-block-post-template.is-flex-container.is-flex-container.columns-2>li,.wp-block-query-loop.is-flex-container.is-flex-container.columns-2>li{width:calc(50% - .625em)}.wp-block-post-template.is-flex-container.is-flex-container.columns-3>li,.wp-block-query-loop.is-flex-container.is-flex-container.columns-3>li{width:calc(33.33333% - .83333em)}.wp-block-post-template.is-flex-container.is-flex-container.columns-4>li,.wp-block-query-loop.is-flex-container.is-flex-container.columns-4>li{width:calc(25% - .9375em)}.wp-block-post-template.is-flex-container.is-flex-container.columns-5>li,.wp-block-query-loop.is-flex-container.is-flex-container.columns-5>li{width:calc(20% - 1em)}.wp-block-post-template.is-flex-container.is-flex-container.columns-6>li,.wp-block-query-loop.is-flex-container.is-flex-container.columns-6>li{width:calc(16.66667% - 1.04167em)}}</style><style id='wp-block-query-pagination-inline-css'>.wp-block-query-pagination>.wp-block-query-pagination-next,.wp-block-query-pagination>.wp-block-query-pagination-numbers,.wp-block-query-pagination>.wp-block-query-pagination-previous{margin-right:.5em;margin-bottom:.5em}.wp-block-query-pagination>.wp-block-query-pagination-next:last-child,.wp-block-query-pagination>.wp-block-query-pagination-numbers:last-child,.wp-block-query-pagination>.wp-block-query-pagination-previous:last-child{margin-right:0}.wp-block-query-pagination .wp-block-query-pagination-previous-arrow{margin-right:1ch;display:inline-block}.wp-block-query-pagination .wp-block-query-pagination-previous-arrow:not(.is-arrow-chevron){transform:scaleX(1)}.wp-block-query-pagination .wp-block-query-pagination-next-arrow{margin-left:1ch;display:inline-block}.wp-block-query-pagination .wp-block-query-pagination-next-arrow:not(.is-arrow-chevron){transform:scaleX(1)}.wp-block-query-pagination.aligncenter{justify-content:center}</style><style id='wp-block-library-inline-css'>:root{--wp-admin-theme-color:#007cba;--wp-admin-theme-color--rgb:0,124,186;--wp-admin-theme-color-darker-10:#006ba1;--wp-admin-theme-color-darker-10--rgb:0,107,161;--wp-admin-theme-color-darker-20:#005a87;--wp-admin-theme-color-darker-20--rgb:0,90,135;--wp-admin-border-width-focus:2px}@media (-webkit-min-device-pixel-ratio:2),(min-resolution:192dpi){:root{--wp-admin-border-width-focus:1.5px}}:root{--wp--preset--font-size--normal:16px;--wp--preset--font-size--huge:42px}:root .has-very-light-gray-background-color{background-color:#eee}:root .has-very-dark-gray-background-color{background-color:#313131}:root .has-very-light-gray-color{color:#eee}:root .has-very-dark-gray-color{color:#313131}:root .has-vivid-green-cyan-to-vivid-cyan-blue-gradient-background{background:linear-gradient(135deg,#00d084,#0693e3)}:root .has-purple-crush-gradient-background{background:linear-gradient(135deg,#34e2e4,#4721fb 50%,#ab1dfe)}:root .has-hazy-dawn-gradient-background{background:linear-gradient(135deg,#faaca8,#dad0ec)}:root .has-subdued-olive-gradient-background{background:linear-gradient(135deg,#fafae1,#67a671)}:root .has-atomic-cream-gradient-background{background:linear-gradient(135deg,#fdd79a,#004a59)}:root .has-nightshade-gradient-background{background:linear-gradient(135deg,#330968,#31cdcf)}:root .has-midnight-gradient-background{background:linear-gradient(135deg,#020381,#2874fc)}.has-regular-font-size{font-size:1em}.has-larger-font-size{font-size:2.625em}.has-normal-font-size{}.has-huge-font-size{}.has-text-align-center{text-align:center}.has-text-align-left{text-align:left}.has-text-align-right{text-align:right}#end-resizable-editor-section{display:none}.aligncenter{clear:both}.items-justified-left{justify-content:flex-start}.items-justified-center{justify-content:center}.items-justified-right{justify-content:flex-end}.items-justified-space-between{justify-content:space-between}.screen-reader-text{border:0;clip:rect(1px,1px,1px,1px);-webkit-clip-path:inset(50%);clip-path:inset(50%);height:1px;margin:-1px;overflow:hidden;padding:0;position:absolute;width:1px;word-wrap:normal !important}.screen-reader-text:focus{background-color:#ddd;clip:auto !important;-webkit-clip-path:none;clip-path:none;color:#444;display:block;font-size:1em;height:auto;left:5px;line-height:normal;padding:15px 23px 14px;text-decoration:none;top:5px;width:auto;z-index:100000}html :where(.has-border-color),html :where([style*=border-width]){border-style:solid}html :where(img[class*=wp-image-]){height:auto;max-width:100%}</style><style id='global-styles-inline-css'>body{--wp--preset--color--black:#000000;--wp--preset--color--cyan-bluish-gray:#abb8c3;--wp--preset--color--white:#ffffff;--wp--preset--color--pale-pink:#f78da7;--wp--preset--color--vivid-red:#cf2e2e;--wp--preset--color--luminous-vivid-orange:#ff6900;--wp--preset--color--luminous-vivid-amber:#fcb900;--wp--preset--color--light-green-cyan:#7bdcb5;--wp--preset--color--vivid-green-cyan:#00d084;--wp--preset--color--pale-cyan-blue:#8ed1fc;--wp--preset--color--vivid-cyan-blue:#0693e3;--wp--preset--color--vivid-purple:#9b51e0;--wp--preset--color--foreground:#000000;--wp--preset--color--background:#ffffff;--wp--preset--color--primary:#1a4548;--wp--preset--color--secondary:#ffe2c7;--wp--preset--color--tertiary:#F6F6F6;--wp--preset--gradient--vivid-cyan-blue-to-vivid-purple:linear-gradient(135deg,rgba(6,147,227,1) 0%,rgb(155,81,224) 100%);--wp--preset--gradient--light-green-cyan-to-vivid-green-cyan:linear-gradient(135deg,rgb(122,220,180) 0%,rgb(0,208,130) 100%);--wp--preset--gradient--luminous-vivid-amber-to-luminous-vivid-orange:linear-gradient(135deg,rgba(252,185,0,1) 0%,rgba(255,105,0,1) 100%);--wp--preset--gradient--luminous-vivid-orange-to-vivid-red:linear-gradient(135deg,rgba(255,105,0,1) 0%,rgb(207,46,46) 100%);--wp--preset--gradient--very-light-gray-to-cyan-bluish-gray:linear-gradient(135deg,rgb(238,238,238) 0%,rgb(169,184,195) 100%);--wp--preset--gradient--cool-to-warm-spectrum:linear-gradient(135deg,rgb(74,234,220) 0%,rgb(151,120,209) 20%,rgb(207,42,186) 40%,rgb(238,44,130) 60%,rgb(251,105,98) 80%,rgb(254,248,76) 100%);--wp--preset--gradient--blush-light-purple:linear-gradient(135deg,rgb(255,206,236) 0%,rgb(152,150,240) 100%);--wp--preset--gradient--blush-bordeaux:linear-gradient(135deg,rgb(254,205,165) 0%,rgb(254,45,45) 50%,rgb(107,0,62) 100%);--wp--preset--gradient--luminous-dusk:linear-gradient(135deg,rgb(255,203,112) 0%,rgb(199,81,192) 50%,rgb(65,88,208) 100%);--wp--preset--gradient--pale-ocean:linear-gradient(135deg,rgb(255,245,203) 0%,rgb(182,227,212) 50%,rgb(51,167,181) 100%);--wp--preset--gradient--electric-grass:linear-gradient(135deg,rgb(202,248,128) 0%,rgb(113,206,126) 100%);--wp--preset--gradient--midnight:linear-gradient(135deg,rgb(2,3,129) 0%,rgb(40,116,252) 100%);--wp--preset--duotone--dark-grayscale:url('#wp-duotone-dark-grayscale');--wp--preset--duotone--grayscale:url('#wp-duotone-grayscale');--wp--preset--duotone--purple-yellow:url('#wp-duotone-purple-yellow');--wp--preset--duotone--blue-red:url('#wp-duotone-blue-red');--wp--preset--duotone--midnight:url('#wp-duotone-midnight');--wp--preset--duotone--magenta-yellow:url('#wp-duotone-magenta-yellow');--wp--preset--duotone--purple-green:url('#wp-duotone-purple-green');--wp--preset--duotone--blue-orange:url('#wp-duotone-blue-orange');--wp--preset--duotone--foreground-and-background:url('#wp-duotone-foreground-and-background');--wp--preset--duotone--foreground-and-secondary:url('#wp-duotone-foreground-and-secondary');--wp--preset--duotone--foreground-and-tertiary:url('#wp-duotone-foreground-and-tertiary');--wp--preset--duotone--primary-and-background:url('#wp-duotone-primary-and-background');--wp--preset--duotone--primary-and-secondary:url('#wp-duotone-primary-and-secondary');--wp--preset--duotone--primary-and-tertiary:url('#wp-duotone-primary-and-tertiary');--wp--preset--font-size--small:1rem;--wp--preset--font-size--medium:1.125rem;--wp--preset--font-size--large:1.75rem;--wp--preset--font-size--x-large:clamp(1.75rem, 3vw, 2.25rem);--wp--preset--font-family--system-font:-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,Oxygen-Sans,Ubuntu,Cantarell,\"Helvetica Neue\",sans-serif;--wp--preset--font-family--source-serif-pro:\"Source Serif Pro\", serif;--wp--custom--spacing--small:max(1.25rem, 5vw);--wp--custom--spacing--large:clamp(4rem, 10vw, 8rem);--wp--custom--typography--font-size--huge:clamp(2.25rem, 4vw, 2.75rem);--wp--custom--typography--font-size--gigantic:clamp(2.75rem, 6vw, 3.25rem);--wp--custom--typography--font-size--colossal:clamp(3.25rem, 8vw, 6.25rem);--wp--custom--typography--line-height--tiny:1.15;--wp--custom--typography--line-height--small:1.2;--wp--custom--typography--line-height--medium:1.4;--wp--custom--typography--line-height--normal:1.6}body{margin:0}body{--wp--style--block-gap:1.5rem}.wp-site-blocks > .alignleft{float:left;margin-right:2em}.wp-site-blocks > .alignright{float:right;margin-left:2em}.wp-site-blocks > .aligncenter{justify-content:center;margin-left:auto;margin-right:auto}.wp-site-blocks > *{margin-block-start:0;margin-block-end:0}.wp-site-blocks > * + *{}h1{font-weight:300;}h2{font-weight:300;}h3{font-weight:300;}h4{font-weight:300;}h5{font-weight:700;text-transform:uppercase}h6{font-weight:400;text-transform:uppercase}a{}.wp-block-button__link{border-radius:0;}.wp-block-post-title{font-weight:300;}.wp-block-post-comments{}.wp-block-pullquote{border-width:1px 0}.wp-block-query-title{font-weight:300;}.wp-block-quote{border-width:1px}.wp-block-site-title{font-style:italic;font-weight:normal;}.has-black-color{}.has-cyan-bluish-gray-color{}.has-white-color{}.has-pale-pink-color{}.has-vivid-red-color{}.has-luminous-vivid-orange-color{}.has-luminous-vivid-amber-color{}.has-light-green-cyan-color{}.has-vivid-green-cyan-color{}.has-pale-cyan-blue-color{}.has-vivid-cyan-blue-color{}.has-vivid-purple-color{}.has-foreground-color{}.has-background-color{}.has-primary-color{}.has-secondary-color{}.has-tertiary-color{}.has-black-background-color{}.has-cyan-bluish-gray-background-color{}.has-white-background-color{}.has-pale-pink-background-color{}.has-vivid-red-background-color{}.has-luminous-vivid-orange-background-color{}.has-luminous-vivid-amber-background-color{}.has-light-green-cyan-background-color{}.has-vivid-green-cyan-background-color{}.has-pale-cyan-blue-background-color{}.has-vivid-cyan-blue-background-color{}.has-vivid-purple-background-color{}.has-foreground-background-color{}.has-background-background-color{}.has-primary-background-color{}.has-secondary-background-color{}.has-tertiary-background-color{}.has-black-border-color{}.has-cyan-bluish-gray-border-color{}.has-white-border-color{}.has-pale-pink-border-color{}.has-vivid-red-border-color{}.has-luminous-vivid-orange-border-color{}.has-luminous-vivid-amber-border-color{}.has-light-green-cyan-border-color{}.has-vivid-green-cyan-border-color{}.has-pale-cyan-blue-border-color{}.has-vivid-cyan-blue-border-color{}.has-vivid-purple-border-color{}.has-foreground-border-color{}.has-background-border-color{}.has-primary-border-color{}.has-secondary-border-color{}.has-tertiary-border-color{}.has-vivid-cyan-blue-to-vivid-purple-gradient-background{}.has-light-green-cyan-to-vivid-green-cyan-gradient-background{}.has-luminous-vivid-amber-to-luminous-vivid-orange-gradient-background{}.has-luminous-vivid-orange-to-vivid-red-gradient-background{}.has-very-light-gray-to-cyan-bluish-gray-gradient-background{}.has-cool-to-warm-spectrum-gradient-background{}.has-blush-light-purple-gradient-background{}.has-blush-bordeaux-gradient-background{}.has-luminous-dusk-gradient-background{}.has-pale-ocean-gradient-background{}.has-electric-grass-gradient-background{}.has-midnight-gradient-background{}.has-vertical-secondary-to-tertiary-gradient-background{}.has-vertical-secondary-to-background-gradient-background{}.has-vertical-tertiary-to-background-gradient-background{}.has-diagonal-primary-to-foreground-gradient-background{}.has-diagonal-secondary-to-background-gradient-background{}.has-diagonal-background-to-secondary-gradient-background{}.has-diagonal-tertiary-to-background-gradient-background{}.has-diagonal-background-to-tertiary-gradient-background{}.has-small-font-size{}.has-medium-font-size{}.has-large-font-size{}.has-x-large-font-size{}.has-system-font-font-family{}.has-source-serif-pro-font-family{}</style><style id='wp-webfonts-inline-css'>@font-face{font-family:\"Source Serif Pro\";font-style:normal;font-weight:200 900;font-display:fallback;src:local(\"Source Serif Pro\"), url('/wp-content/themes/twentytwentytwo/assets/fonts/source-serif-pro/SourceSerif4Variable-Roman.ttf.woff2') format('woff2');font-stretch:normal}@font-face{font-family:\"Source Serif Pro\";font-style:italic;font-weight:200 900;font-display:fallback;src:local(\"Source Serif Pro\"), url('/wp-content/themes/twentytwentytwo/assets/fonts/source-serif-pro/SourceSerif4Variable-Italic.ttf.woff2') format('woff2');font-stretch:normal}</style><link rel='stylesheet' id='twentytwentytwo-style-css' href='https://wordpress.com/wp-content/themes/twentytwentytwo/style.css?ver=1.2' media='all' /><script src='https://wordpress.com/wp-includes/blocks/navigation/view.min.js?ver=009e29110e016c14bac4ba0ecc809fcd' id='wp-block-navigation-view-js'></script><link rel=\"https://api.w.org/\" href=\"https://wordpress.com/index.php?rest_route=/\" /><link rel=\"EditURI\" type=\"application/rsd+xml\" title=\"RSD\" href=\"https://wordpress.com/xmlrpc.php?rsd\" /><link rel=\"wlwmanifest\" type=\"application/wlwmanifest+xml\" href=\"https://wordpress.com/wp-includes/wlwmanifest.xml\" /><meta name=\"generator\" content=\"WordPress 6.0\" /><style>.wp-elements-c2d3692c067254e99911402d49af8a7d a{}</style><style>.wp-container-1{display:flex;flex-wrap:wrap;align-items:center}.wp-container-1 > *{margin:0}</style><style>.wp-container-3{display:flex;flex-wrap:wrap;align-items:center;justify-content:flex-end}.wp-container-3 > *{margin:0}</style><style>.wp-container-4{display:flex;flex-wrap:wrap;align-items:center;justify-content:space-between}.wp-container-4 > *{margin:0}</style><style>.wp-container-5 > :where(:not(.alignleft):not(.alignright)){max-width:650px;margin-left:auto !important;margin-right:auto !important}.wp-container-5 > .alignwide{max-width:1000px}.wp-container-5 .alignfull{max-width:none}.wp-container-5 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-5 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-5 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-5 > *{margin-block-start:0;margin-block-end:0}.wp-container-5 > * + *{margin-block-end:0}</style><style>.wp-container-6 > :where(:not(.alignleft):not(.alignright)){max-width:650px;margin-left:auto !important;margin-right:auto !important}.wp-container-6 > .alignwide{max-width:1000px}.wp-container-6 .alignfull{max-width:none}.wp-container-6 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-6 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-6 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-6 > *{margin-block-start:0;margin-block-end:0}.wp-container-6 > * + *{margin-block-end:0}</style><style>.wp-container-7 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-7 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-7 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-7 > *{margin-block-start:0;margin-block-end:0}.wp-container-7 > * + *{margin-block-end:0}</style><style>.wp-container-8 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-8 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-8 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-8 > *{margin-block-start:0;margin-block-end:0}.wp-container-8 > * + *{margin-block-end:0}</style><style>.wp-container-9{display:flex;flex-wrap:nowrap;align-items:center}.wp-container-9 > *{margin:0}</style><style>.wp-container-10 > :where(:not(.alignleft):not(.alignright)){max-width:650px;margin-left:auto !important;margin-right:auto !important}.wp-container-10 > .alignwide{max-width:1000px}.wp-container-10 .alignfull{max-width:none}.wp-container-10 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-10 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-10 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-10 > *{margin-block-start:0;margin-block-end:0}.wp-container-10 > * + *{margin-block-end:0}</style><style>.wp-container-11 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-11 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-11 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-11 > *{margin-block-start:0;margin-block-end:0}.wp-container-11 > * + *{margin-block-end:0}</style><style>.wp-container-12 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-12 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-12 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-12 > *{margin-block-start:0;margin-block-end:0}.wp-container-12 > * + *{margin-block-end:0}</style><style>.wp-container-13{display:flex;flex-wrap:wrap;align-items:center;justify-content:space-between}.wp-container-13 > *{margin:0}</style><style>.wp-container-14 > :where(:not(.alignleft):not(.alignright)){max-width:650px;margin-left:auto !important;margin-right:auto !important}.wp-container-14 > .alignwide{max-width:1000px}.wp-container-14 .alignfull{max-width:none}.wp-container-14 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-14 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-14 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-14 > *{margin-block-start:0;margin-block-end:0}.wp-container-14 > * + *{margin-block-end:0}</style><style>.wp-container-15{display:flex;flex-wrap:wrap;align-items:center;justify-content:space-between}.wp-container-15 > *{margin:0}</style><style>.wp-container-16 > :where(:not(.alignleft):not(.alignright)){max-width:650px;margin-left:auto !important;margin-right:auto !important}.wp-container-16 > .alignwide{max-width:1000px}.wp-container-16 .alignfull{max-width:none}.wp-container-16 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-16 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-16 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-16 > *{margin-block-start:0;margin-block-end:0}.wp-container-16 > * + *{margin-block-end:0}</style><style>.wp-container-17 > :where(:not(.alignleft):not(.alignright)){max-width:650px;margin-left:auto !important;margin-right:auto !important}.wp-container-17 > .alignwide{max-width:1000px}.wp-container-17 .alignfull{max-width:none}.wp-container-17 > .alignleft{float:left;margin-inline-start:0;margin-inline-end:2em}.wp-container-17 > .alignright{float:right;margin-inline-start:2em;margin-inline-end:0}.wp-container-17 > .aligncenter{margin-left:auto !important;margin-right:auto !important}.wp-container-17 > *{margin-block-start:0;margin-block-end:0}.wp-container-17 > * + *{margin-block-end:0}</style></head><body class=\"home blog wp-embed-responsive\"><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-dark-grayscale\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0 0.49803921568627\" /><feFuncG type=\"table\" tableValues=\"0 0.49803921568627\" /><feFuncB type=\"table\" tableValues=\"0 0.49803921568627\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-grayscale\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0 1\" /><feFuncG type=\"table\" tableValues=\"0 1\" /><feFuncB type=\"table\" tableValues=\"0 1\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-purple-yellow\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0.54901960784314 0.98823529411765\" /><feFuncG type=\"table\" tableValues=\"0 1\" /><feFuncB type=\"table\" tableValues=\"0.71764705882353 0.25490196078431\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-blue-red\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0 1\" /><feFuncG type=\"table\" tableValues=\"0 0.27843137254902\" /><feFuncB type=\"table\" tableValues=\"0.5921568627451 0.27843137254902\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-midnight\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0 0\" /><feFuncG type=\"table\" tableValues=\"0 0.64705882352941\" /><feFuncB type=\"table\" tableValues=\"0 1\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-magenta-yellow\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0.78039215686275 1\" /><feFuncG type=\"table\" tableValues=\"0 0.94901960784314\" /><feFuncB type=\"table\" tableValues=\"0.35294117647059 0.47058823529412\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-purple-green\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0.65098039215686 0.40392156862745\" /><feFuncG type=\"table\" tableValues=\"0 1\" /><feFuncB type=\"table\" tableValues=\"0.44705882352941 0.4\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-blue-orange\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0.098039215686275 1\" /><feFuncG type=\"table\" tableValues=\"0 0.66274509803922\" /><feFuncB type=\"table\" tableValues=\"0.84705882352941 0.41960784313725\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-foreground-and-background\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0 1\" /><feFuncG type=\"table\" tableValues=\"0 1\" /><feFuncB type=\"table\" tableValues=\"0 1\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-foreground-and-secondary\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0 1\" /><feFuncG type=\"table\" tableValues=\"0 0.88627450980392\" /><feFuncB type=\"table\" tableValues=\"0 0.78039215686275\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-foreground-and-tertiary\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0 0.96470588235294\" /><feFuncG type=\"table\" tableValues=\"0 0.96470588235294\" /><feFuncB type=\"table\" tableValues=\"0 0.96470588235294\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-primary-and-background\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0.10196078431373 1\" /><feFuncG type=\"table\" tableValues=\"0.27058823529412 1\" /><feFuncB type=\"table\" tableValues=\"0.28235294117647 1\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-primary-and-secondary\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0.10196078431373 1\" /><feFuncG type=\"table\" tableValues=\"0.27058823529412 0.88627450980392\" /><feFuncB type=\"table\" tableValues=\"0.28235294117647 0.78039215686275\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 0 0\" width=\"0\" height=\"0\" focusable=\"false\" role=\"none\" style=\"visibility:hidden;position:absolute;left:-9999px;overflow:hidden\"><defs><filter id=\"wp-duotone-primary-and-tertiary\"><feColorMatrix color-interpolation-filters=\"sRGB\" type=\"matrix\" values=\" .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 .299 .587 .114 0 0 \" /><feComponentTransfer color-interpolation-filters=\"sRGB\"><feFuncR type=\"table\" tableValues=\"0.10196078431373 0.96470588235294\" /><feFuncG type=\"table\" tableValues=\"0.27058823529412 0.96470588235294\" /><feFuncB type=\"table\" tableValues=\"0.28235294117647 0.96470588235294\" /><feFuncA type=\"table\" tableValues=\"1 1\" /></feComponentTransfer><feComposite in2=\"SourceGraphic\" operator=\"in\" /></filter></defs></svg><div class=\"wp-site-blocks\"><header class=\"wp-block-template-part\"><div class=\"wp-container-6 wp-elements-c2d3692c067254e99911402d49af8a7d wp-block-group alignfull has-background-color has-foreground-background-color has-text-color has-background has-link-color\" style=\"padding-top:0px;padding-bottom:0px\"><header class=\"alignwide wp-block-template-part\"><div class=\"wp-container-5 wp-block-group\"><div class=\"wp-container-4 wp-block-group alignwide\" style=\"\"><div class=\"wp-container-1 wp-block-group\"><h1 class=\"wp-block-site-title\"><a href=\"https://wordpress.com\" rel=\"home\" aria-current=\"page\">test</a></h1></div><nav class=\"wp-container-3 is-responsive items-justified-right wp-block-navigation\" aria-label=\"\"><button aria-haspopup=\"true\" aria-label=\"Open menu\" class=\"wp-block-navigation__responsive-container-open \" data-micromodal-trigger=\"modal-2\"><svg width=\"24\" height=\"24\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\" aria-hidden=\"true\" focusable=\"false\"><rect x=\"4\" y=\"7.5\" width=\"16\" height=\"1.5\" /><rect x=\"4\" y=\"15\" width=\"16\" height=\"1.5\" /></svg></button><div class=\"wp-block-navigation__responsive-container \" style=\"\" id=\"modal-2\"><div class=\"wp-block-navigation__responsive-close\" tabindex=\"-1\" data-micromodal-close><div class=\"wp-block-navigation__responsive-dialog\" aria-label=\"Menu\"><button aria-label=\"Close menu\" data-micromodal-close class=\"wp-block-navigation__responsive-container-close\"><svg xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\" width=\"24\" height=\"24\" aria-hidden=\"true\" focusable=\"false\"><path d=\"M13 11.8l6.1-6.3-1-1-6.1 6.2-6.1-6.2-1 1 6.1 6.3-6.5 6.7 1 1 6.5-6.6 6.5 6.6 1-1z\"></path></svg></button><div class=\"wp-block-navigation__responsive-container-content\" id=\"modal-2-content\"><ul class=\"wp-block-page-list\"><li class=\"wp-block-pages-list__item wp-block-navigation-item open-on-hover-click\"><a class=\"wp-block-pages-list__item__link wp-block-navigation-item__content\" href=\"https://wordpress.com/?page_id=2\">Sample Page</a></li></ul></div></div></div></div></nav></div></div></header><figure class=\"wp-block-image alignwide size-full\"><img src=\"https://wordpress.com/wp-content/themes/twentytwentytwo/assets/images/flight-path-on-transparent-d.png\" alt=\"Illustration of a bird flying.\"/></figure></div><div style=\"height:66px\" aria-hidden=\"true\" class=\"wp-block-spacer\"></div></header><main class=\"wp-container-14 wp-block-query\"><ul class=\"wp-container-12 alignwide wp-block-post-template\"><li class=\"wp-block-post post-1 post type-post status-publish format-standard hentry category-uncategorized\"><div class=\"wp-container-11 wp-container-10 wp-block-group\"><h2 class=\"alignwide wp-block-post-title has-var-wp-custom-typography-font-size-huge-clamp-2-25-rem-4-vw-2-75-rem-font-size\"><a href=\"https://wordpress.com/?p=1\" target=\"_self\" rel=\"\">Hello world!</a></h2><div class=\"wp-container-9 wp-block-columns alignwide\"><div class=\"wp-container-7 wp-block-column\" style=\"flex-basis:650px\"><div class=\"wp-block-post-excerpt\"><p class=\"wp-block-post-excerpt__excerpt\">Welcome to WordPress. This is your first post. Edit or delete it, then start writing! </p></div><div style=\"font-style:italic;font-weight:400\" class=\"wp-block-post-date has-small-font-size\"><time datetime=\"2022-05-29T14:32:06+00:00\"><a href=\"https://wordpress.com/?p=1\">May 29, 2022</a></time></div></div><div class=\"wp-container-8 wp-block-column\"></div></div><div style=\"height:112px\" aria-hidden=\"true\" class=\"wp-block-spacer\"></div></div></li></ul></main><footer class=\"wp-block-template-part\"><div class=\"wp-container-17 wp-block-group\" style=\"\"><div class=\"wp-container-16 wp-block-group alignfull\"><div class=\"wp-container-15 wp-block-group alignwide\" style=\"padding-top:4rem;padding-bottom:4rem\"><p class=\"wp-block-site-title\"><a href=\"https://wordpress.com\" rel=\"home\" aria-current=\"page\">test</a></p><p class=\"has-text-align-right\">Proudly powered by <a href=\"https://wordpress.org\" rel=\"nofollow\">WordPress</a></p></div></div></div></footer></div><style id=\"skip-link-styles\">.skip-link.screen-reader-text{border:0;clip:rect(1px,1px,1px,1px);clip-path:inset(50%);height:1px;margin:-1px;overflow:hidden;padding:0;position:absolute !important;width:1px;word-wrap:normal !important}.skip-link.screen-reader-text:focus{background-color:#eee;clip:auto !important;clip-path:none;color:#444;display:block;font-size:1em;height:auto;left:5px;line-height:normal;padding:15px 23px 14px;text-decoration:none;top:5px;width:auto;z-index:100000}</style><script>(function(){var e=document.querySelector('main'),n,c,o;</script></body></html>"
    headers:
      - "Content-Type: text/html"
      - "Server: Apache/2.4.53 (Debian)"
      - "X-Powered-By: PHP/7.4.29"
    statusCode: 200
  - regex: "^(wp-login.php|/wp-admin)$"
    handler: "<!DOCTYPE html><html lang=\"en-US\"><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" /><title>Log In ‹ test — WordPress</title><meta name='robots' content='max-image-preview:large, noindex, noarchive' /><link rel='dns-prefetch' href='//s.w.org' /><link rel='stylesheet' id='dashicons-css' href='https://wordpress.com/wp-includes/css/dashicons.min.css?ver=6.0' media='all' /><link rel='stylesheet' id='buttons-css' href='https://wordpress.com/wp-includes/css/buttons.min.css?ver=6.0' media='all' /><link rel='stylesheet' id='forms-css' href='https://wordpress.com/wp-admin/css/forms.min.css?ver=6.0' media='all' /><link rel='stylesheet' id='l10n-css' href='https://wordpress.com/wp-admin/css/l10n.min.css?ver=6.0' media='all' /><link rel='stylesheet' id='login-css' href='https://wordpress.com/wp-admin/css/login.min.css?ver=6.0' media='all' /><meta name='referrer' content='strict-origin-when-cross-origin' /><meta name=\"viewport\" content=\"width=device-width\" /></head><body class=\"login no-js login-action-login wp-core-ui locale-en-us\"><script type=\"text/javascript\">;document.body.className=document.body.className.replace('no-js','js');</script><div id=\"login\"><h1><a href=\"https://wordpress.org/\">Powered by WordPress</a></h1><form action=\"\" method=\"post\"><p><label for=\"user_login\">Username or Email Address</label><input type=\"text\" name=\"log\" id=\"user_login\" class=\"input\" value=\"\" size=\"20\" autocapitalize=\"off\" autocomplete=\"username\" /></p><div class=\"user-pass-wrap\"><label for=\"user_pass\">Password</label><div class=\"wp-pwd\"><input type=\"password\" name=\"pwd\" id=\"user_pass\" class=\"input password-input\" value=\"\" size=\"20\" autocomplete=\"current-password\" /><button type=\"button\" class=\"button button-secondary wp-hide-pw hide-if-no-js\" data-toggle=\"0\" aria-label=\"Show password\"><span class=\"dashicons dashicons-visibility\" aria-hidden=\"true\"></span></button></div></div><p class=\"forgetmenot\"><input name=\"rememberme\" type=\"checkbox\" id=\"rememberme\" value=\"forever\" /><label for=\"rememberme\">Remember Me</label></p><p class=\"submit\"><input type=\"submit\" name=\"wp-submit\" id=\"wp-submit\" class=\"button button-primary button-large\" value=\"Log In\" /><input type=\"hidden\" name=\"redirect_to\" value=\"https://wordpress.com/wp-admin/\" /><input type=\"hidden\" name=\"testcookie\" value=\"1\" /></p></form><p id=\"nav\"><a href=\"https://wordpress.com/wp-login.php?action=lostpassword\">Lost your password?</a></p><script type=\"text/javascript\">;function wp_attempt_focus(){setTimeout(function(){try{d=document.getElementById('user_login');d.focus();d.select()}catch(t){}},200)};wp_attempt_focus();if(typeof wpOnload==='function'){wpOnload()};</script><p id=\"backtoblog\"><a href=\"https://wordpress.com/\">← Go to test</a></p></div><script src='https://wordpress.com/wp-includes/js/jquery/jquery.min.js?ver=3.6.0' id='jquery-core-js'></script><script src='https://wordpress.com/wp-includes/js/jquery/jquery-migrate.min.js?ver=3.3.2' id='jquery-migrate-js'></script><script id='zxcvbn-async-js-extra'> var _zxcvbnSettings = {\"src\":\"http:\\/\\/X\\/wp-includes\\/js\\/zxcvbn.min.js\"}; </script><script src='https://wordpress.com/wp-includes/js/zxcvbn-async.min.js?ver=1.0' id='zxcvbn-async-js'></script><script src='https://wordpress.com/wp-includes/js/dist/vendor/regenerator-runtime.min.js?ver=0.13.9' id='regenerator-runtime-js'></script><script src='https://wordpress.com/wp-includes/js/dist/vendor/wp-polyfill.min.js?ver=3.15.0' id='wp-polyfill-js'></script><script src='https://wordpress.com/wp-includes/js/dist/hooks.min.js?ver=c6d64f2cb8f5c6bb49caca37f8828ce3' id='wp-hooks-js'></script><script src='https://wordpress.com/wp-includes/js/dist/i18n.min.js?ver=ebee46757c6a411e38fd079a7ac71d94' id='wp-i18n-js'></script><script id='wp-i18n-js-after'> wp.i18n.setLocaleData( { 'text direction\\u0004ltr': [ 'ltr' ] } ); </script><script id='password-strength-meter-js-extra'> var pwsL10n = {\"unknown\":\"Password strength unknown\",\"short\":\"Very weak\",\"bad\":\"Weak\",\"good\":\"Medium\",\"strong\":\"Strong\",\"mismatch\":\"Mismatch\"}; </script><script src='https://wordpress.com/wp-admin/js/password-strength-meter.min.js?ver=6.0' id='password-strength-meter-js'></script><script src='https://wordpress.com/wp-includes/js/underscore.min.js?ver=1.13.3' id='underscore-js'></script><script id='wp-util-js-extra'> var _wpUtilSettings = {\"ajax\":{\"url\":\"\\/wp-admin\\/admin-ajax.php\"}}; </script><script src='https://wordpress.com/wp-includes/js/wp-util.min.js?ver=6.0' id='wp-util-js'></script><script id='user-profile-js-extra'> var userProfileL10n = {\"user_id\":\"0\",\"nonce\":\"4deb99ab74\"}; </script><script src='https://wordpress.com/wp-admin/js/user-profile.min.js?ver=6.0' id='user-profile-js'></script><div class=\"clear\"></div></body></html>"
    headers:
      - "Content-Type: text/html"
      - "Server: Apache/2.4.53 (Debian)"
      - "X-Powered-By: PHP/7.4.29"
    statusCode: 200
 ```

![alt text](https://i.postimg.cc/529V6jYz/Schermata-2022-06-02-alle-12-42-46.png)


### Example HTTP Honeypot on 8080 port

###### http-8080.yaml

```yaml
apiVersion: "v1"
protocol: "http"
address: ":8080"
description: "Apache 401"
commands:
  - regex: ".*"
    handler: "Unauthorized"
    headers:
      - "www-Authenticate: Basic"
      - "server: Apache"
    statusCode: 401
 ```

![alt text](https://i.postimg.cc/T1cs6qc4/Schermata-2022-06-02-alle-12-43-55.png)

### Example SSH Honeypot

###### ssh-22.yaml

```yaml
apiVersion: "v1"
protocol: "ssh"
address: ":22"
description: "SSH interactive"
commands:
  - regex: "^ls$"
    handler: "Documents Images  Desktop Downloads .m2 .kube .ssh  .docker"
  - regex: "^pwd$"
    handler: "/home/"
  - regex: "^uname -m$"
    handler: "x86_64"
  - regex: "^docker ps$"
    handler: "CONTAINER ID   IMAGE     COMMAND   CREATED   STATUS    PORTS     NAMES"
  - regex: "^docker .*$"
    handler: "Error response from daemon: dial unix docker.raw.sock: connect: connection refused"
  - regex: "^uname$"
    handler: "Linux"
  - regex: "^ps$"
    handler: "  PID TTY           TIME CMD\n21642 ttys000    0:00.07 /bin/dockerd"
  - regex: "^(.+)$"
    handler: "command not found"
serverVersion: "OpenSSH"
serverName: "ubuntu"
passwordRegex: "^(root|qwerty|Smoker666)$"
deadlineTimeoutSeconds: 60
 ```

![alt text](https://i.postimg.cc/jdpfT0LB/Schermata-2022-06-02-alle-12-46-50.png)

## Features

- SSH Honeypot
- HTTP Honeypot
- TCP Honeypot
- Easy to create a new strategy
- Easy to extend event tracking logic
- Strong code quality
- Docker
- RabbitMQ integration

## TODO

- telnet
- UDP

# ROADMAP

- SaaS Platform


## Documentation

- [API Docs](https://) #TODO

## Contributing

The beelzebub team enthusiastically welcomes contributions and project participation! There's a bunch of things you can do if you want to contribute! The [Contributor Guide](CONTRIBUTING.md) has all the information you need for everything from reporting bugs to contributing entire new features. Please don't hesitate to jump in if you'd like to, or even ask us questions if something isn't clear.

All participants and maintainers in this project are expected to follow [Code of Conduct](CODE_OF_CONDUCT.md), and just generally be excellent to each other.

Happy hacking!

## License

This project is licensed under [GNU GPL 3 License](LICENSE).
